@@ -8,14 +8,15 @@ use Net::XMPP2::Util qw/split_jid/;
 use Net::XMPP2::Event;
 use Net::XMPP2::SimpleConnection;
 use Net::XMPP2::Namespaces qw/xmpp_ns/;
+use Net::XMPP2::Extendable;
 use Net::XMPP2::Error;
 use Net::DNS;
 
-our @ISA = qw/Net::XMPP2::SimpleConnection Net::XMPP2::Event/;
+our @ISA = qw/Net::XMPP2::SimpleConnection Net::XMPP2::Event Net::XMPP2::Extendable/;
 
 =head1 NAME
 
-Net::XMPP2::Connection - A XML stream that implements the XMPP RFC 3920.
+Net::XMPP2::Connection - XML stream that implements the XMPP RFC 3920.
 
 =head1 SYNOPSIS
 
@@ -63,15 +64,6 @@ Please look in RFC 3066 how C<$tag> should look like.
 This can be used to set the settings C<username>, C<domain>
 (and optionally C<resource>) from a C<$jid>.
 
-=item register => $mode
-
-If this settings is given this connection will attempt to register
-an account in band on the server if C<$mode> is 'auto'.
-
-If C<$mode> is 'manual' the event C<in_band_register_form> will be
-emitted (see also EVENTS documentation about the arguments of that
-event, and how to continue the login procedure).
-
 =item resource => $resource
 
 If this argument is given C<$resource> will be passed as desired
@@ -127,8 +119,7 @@ If C<$bool> is true no SSL will be used.
 sub new {
    my $this = shift;
    my $class = ref($this) || $this;
-   my $self = {  language => 'en', @_ };
-   bless $self, $class;
+   my $self = $class->SUPER::new (language => 'en', @_);
 
    $self->{parser} = new Net::XMPP2::Parser;
    $self->{writer} = Net::XMPP2::Writer->new (
@@ -139,8 +130,18 @@ sub new {
       $self->handle_stanza (@_);
    });
    $self->{parser}->set_error_cb (sub {
-      $self->event (xml_parser_error => $_[0], $_[1]);
-      $self->disconnect ("xml error: $_[0], $_[1]");
+      my ($ex, $data, $type) = @_;
+      if ($type eq 'xml') {
+         my $pe = Net::XMPP2::Error::Parser->new (exception => $_[0], data => $_[1]);
+         $self->event (xml_parser_error => $pe);
+         $self->disconnect ("xml error: $_[0], $_[1]");
+      } else {
+         my $pe = Net::XMPP2::Error->new (
+            text => "uncaught exception in stanza handling: $ex"
+         );
+         $self->event (uncaught_exception_error => $pe);
+         $self->disconnect ($pe->string);
+      }
    });
 
    $self->{iq_id}              = 1;
@@ -165,6 +166,35 @@ sub new {
          unless $self->{$_};
    }
 
+   my $proxy_cb = sub {
+      my ($self, $er) = @_;
+      $self->event (error => $er);
+      1
+   };
+
+   $self->reg_cb (
+      xml_parser_error => $proxy_cb,
+      sasl_error       => $proxy_cb,
+      stream_error     => $proxy_cb,
+      bind_error       => $proxy_cb,
+      iq_result_cb_exception => sub {
+         my ($self, $ex) = @_;
+         $self->event (error =>
+            Net::XMPP2::Error::Exception->new (
+               exception => $ex, context => 'iq result callback execution'
+            )
+         );
+         1
+      },
+      tls_error => sub {
+         my ($self) = @_;
+         $self->event (error =>
+            Net::XMPP2::Error->new (text => 'tls_error: tls negotiation failed')
+         );
+         1
+      },
+   );
+
    return $self;
 }
 
@@ -276,10 +306,12 @@ sub handle_stanza {
       return;
    }
 
+   $self->event (recv_stanza_xml => $node);
+
    if ($node->eq (stream => 'features')) {
       $self->event (stream_features => $node);
-      $self->handle_stream_features ($node);
       $self->{features} = $node;
+      $self->handle_stream_features ($node);
 
    } elsif ($node->eq (tls => 'proceed')) {
       $self->enable_ssl;
@@ -300,8 +332,10 @@ sub handle_stanza {
    } elsif ($node->eq (sasl => 'failure')) {
       my $error = Net::XMPP2::Error::SASL->new (node => $node);
       $self->event (sasl_error => $error);
+      $self->disconnect ('SASL authentication failure: ' . $error->string);
 
    } elsif ($node->eq (client => 'iq')) {
+      $self->event (iq_xml => $node);
       $self->handle_iq ($node);
 
    } elsif ($node->eq (client => 'message')) {
@@ -387,7 +421,7 @@ sub send_iq {
    my $timeout = delete $attrs{timeout} || $self->{default_iq_timeout};
    if ($timeout) {
       $self->{iq_timers}->{$id} =
-         AnyEvent->timer (after => $timeout, sub {
+         AnyEvent->timer (after => $timeout, cb => sub {
             delete $self->{iq_timers}->{$id};
             my $cb = delete $self->{iqs}->{$id};
             $cb->(undef, Net::XMPP2::Error::IQ->new)
@@ -453,7 +487,10 @@ sub handle_iq {
 
    if ($type eq 'result') {
       if (my $cb = delete $self->{iqs}->{$id}) {
-         $cb->($node);
+         eval {
+            $cb->($node);
+         };
+         if ($@) { $self->event (iq_result_cb_exception => $@) }
       }
 
    } elsif ($type eq 'error') {
@@ -484,80 +521,10 @@ sub send_sasl_auth {
    );
 }
 
-=item B<request_inband_register_form ($finish_cb)>
-
-This method starts a in-band-registration attempt.  When finished C<$finish_cb>
-will be called with the first argument being a L<Net::XMPP2::Ext::RegisterForm>
-object (will be undef if an error occured) and the second an optional error
-object of type L<Net::XMPP2::Error::Register> if an error occured.
-
-=cut
-
-sub request_inband_register_form {
-   my ($self, $finish_cb) = @_;
-
-   $self->send_iq (
-      get =>
-         sub {
-            my ($w) = @_;
-            $w->addPrefix (xmpp_ns ('register'), '');
-            $w->emptyTag ([qw/register query/]);
-         },
-         sub {
-            my ($node, $error) = @_;
-            my $form;
-            $form = Net::XMPP2::Ext::RegisterForm (node => $node, connection => $self)
-               unless $error;
-            $finish_cb->($form, $error);
-         }
-   );
-}
-
-sub do_auto_register {
-   my ($self, $mechs) = @_;
-
-   $self->request_inband_register_form (sub {
-      my ($form, $error) = @_;
-
-      if ($error) {
-         $self->event (in_band_register_error => $error);
-
-      } else {
-         if ($self->{register} ne 'manual') {
-            # of course this blows up if the form was more complicated
-            # any ideas?
-            $form->auto_submit (
-               username => $self->{username},
-               password => $self->{password},
-               cb => sub {
-                  my ($form, $error) = @_;
-                  if ($error) {
-                     $self->event (auto_in_band_register_error => $error);
-                  } else {
-                     $self->event ('auto_in_band_register_ok');
-                     $self->send_sasl_auth (@$mechs) if @$mechs;
-                  }
-               }
-            );
-
-         } else {
-            $self->event (
-               in_band_register_form =>
-                  $form,
-                  sub { $self->send_sasl_auth (@$mechs) if @$mechs }
-            )
-         }
-      }
-   });
-
-}
-
 sub handle_stream_features {
    my ($self, $node) = @_;
-   my @mechs = $node->find_all ([qw/sasl mechanisms/], [qw/sasl mechanism/]);
    my @bind  = $node->find_all ([qw/bind bind/]);
    my @tls   = $node->find_all ([qw/tls starttls/]);
-   my @iqa   = $node->find_all ([qw/iqauth auth/]);
 
    # and yet another weird thingie: in XEP-0077 it's said that
    # the register feature MAY be advertised by the server. That means:
@@ -569,12 +536,10 @@ sub handle_stream_features {
       $self->{writer}->send_starttls;
 
    } elsif (not $self->{authenticated}) {
-      if ($self->{register}) {
-         $self->do_auto_register (\@mechs);
-      } elsif (@mechs) {
-         $self->send_sasl_auth (@mechs)
-      } elsif (@iqa) {
-         $self->do_iq_auth;
+      my $continue = 1;
+      $self->event (stream_pre_authentication => \$continue);
+      if ($continue) {
+         $self->authenticate;
       }
 
    } elsif (@bind) {
@@ -582,6 +547,30 @@ sub handle_stream_features {
    }
 }
 
+=item B<authenticate>
+
+This method should be called after the C<stream_pre_authentication> event
+was emitted to continue authentication of the stream.
+
+Usually this method only has to be called when you want to register before
+you authenticate. See also the documentation of the C<stream_pre_authentication>
+event below.
+
+=cut
+
+sub authenticate {
+   my ($self) = @_;
+   my $node = $self->{features};
+   my @mechs = $node->find_all ([qw/sasl mechanisms/], [qw/sasl mechanism/]);
+   my @iqa   = $node->find_all ([qw/iqauth auth/]);
+
+   if (@mechs) {
+      $self->send_sasl_auth (@mechs)
+   } elsif (@iqa) {
+      $self->do_iq_auth;
+   }
+}
+
 sub handle_sasl_challenge {
    my ($self, $node) = @_;
    $self->{writer}->send_sasl_response ($node->text);
@@ -678,8 +667,13 @@ sub do_rebind {
             my ($ret_iq, $error) = @_;
 
             if ($error) {
-               my ($res) = $error->xml_node ()->find_all ([qw/bind bind/], [qw/bind resource/]);
-               $self->event (bind_error => $error, ($res ? $res : $self->{resource}));
+               # TODO: make bind error into a seperate error class?
+               if ($error->xml_node ()) {
+                  my ($res) = $error->xml_node ()->find_all ([qw/bind bind/], [qw/bind resource/]);
+                  $self->event (bind_error => $error, ($res ? $res : $self->{resource}));
+               } else {
+                  $self->event (bind_error => $error);
+               }
 
             } else {
                my @jid = $ret_iq->find_all ([qw/bind bind/], [qw/bind jid/]);
@@ -723,6 +717,18 @@ These events can be registered on with C<reg_cb>:
 This event is sent when a stream feature (<features>) tag is received. C<$node> is the
 L<Net::XMPP2::Node> object that represents the <features> tag.
 
+=item stream_pre_authentication => $rcontinue
+
+This event is emitted after TLS/SSL was initiated (if enabled) and before any
+authentication happened. C<$rcontinue> is a reference to a scalar that per default
+holds a true value. If that scalar is true the authentication will continue
+after handling this event. If you set C<$$rcontinue> to a false value
+the authentication will stop and you have to call the C<authenticate>
+method later.
+
+This event is usually used when you want to do in-band registration,
+see also L<Net::XMPP2::Ext::Registration>.
+
 =item stream_ready => $jid
 
 This event is sent if the XML stream has been established (and
@@ -730,11 +736,26 @@ resources have been bound) and is ready for transmitting regular stanzas.
 
 C<$jid> is the bound jabber id.
 
+=item error => $error
+
+This event is generated whenever some error occured.
+C<$error> is an instance of L<Net::XMPP2::Error>.
+Trivial error reporting may look like this:
+
+   $con->reg_cb (error => sub { warn "xmpp error: " . $_[1]->string . "\n"; 1 });
+
+Basically this event is a collect event for all other error events.
+
 =item stream_error => $error
 
 This event is sent if a XML stream error occured. C<$error>
 is a L<Net::XMPP2::Error::Stream> object.
 
+=item xml_parser_error => $error
+
+This event is generated whenever the parser trips over XML that it can't
+read. C<$error> is a L<Net::XMPP2::Error::Parser> object.
+
 =item tls_error
 
 This event is emitted when a TLS error occured on TLS negotiation.
@@ -779,6 +800,41 @@ C<$host> and C<$port> were the host and port we were connected to.
 Note: C<$host> and C<$port> might be different from the domain you passed to
 C<new> if C<connect> performed a SRV RR lookup.
 
+=item recv_stanza_xml => $node
+
+This event is generated before any processing of a "XML" stanza happens.
+C<$node> is the node of the stanza that is being processed, it's of
+type L<Net::XMPP2::Node>.
+
+This method might not be as handy for debuggin purposes as C<debug_recv>.
+
+=item send_stanza_data => $data
+
+This event is generated shortly before data is sent to the socket.
+C<$data> contains a complete "XML" stanza or the end of stream closing
+tag. This method is useful for debugging purposes and I recommend
+using XML::Twig or something like that to display it nicely.
+
+See also the event C<debug_send>.
+
+=item debug_send => $data
+
+This method is invoked whenever data is written out. This event
+is mostly the same as C<send_stanza_data>.
+
+=item debug_recv => $data
+
+This method is incoked whenever a chunk of data was received.
+
+It works to filter C<$data> through L<XML::Twig> for debugging
+display purposes sometimes, but as C<$data> is some arbitrary chunk
+of bytes you might get a XML parse error (did I already mention that XMPP's
+application of "XML" sucks?).
+
+So you might want to use C<recv_stanza_xml> to detect
+complete stanzas. Unfortunately C<recv_stanza_xml> doesn't have the
+bytes anymore and just a datastructure (L<Net::XMPP2::Node>).
+
 =item presence_xml => $node
 
 This event is sent when a presence stanza is received. C<$node> is the
@@ -789,6 +845,11 @@ L<Net::XMPP2::Node> object that represents the <presence> tag.
 This event is sent when a message stanza is received. C<$node> is the
 L<Net::XMPP2::Node> object that represents the <message> tag.
 
+=item iq_xml => $node
+
+This event is emitted when a iq stanza arrives. C<$node> is the
+L<Net::XMPP2::Node> object that represents the <iq> tag.
+
 =item iq_set_request_xml => $node, $handled_ref
 
 =item iq_get_request_xml => $node, $handled_ref
@@ -803,6 +864,11 @@ If one of the event handlers handled this message the scalar pointed at by
 the reference in C<$handled_ref> should be set to 1 true value. If C<$$handled_ref>
 is still false after all event handlers were executed an error iq will be generated.
 
+=item iq_result_cb_exception => $exception
+
+If the C<$result_cb> of a C<send_iq> operation somehow threw a exception
+or failed this event will be generated.
+
 =back
 
 =head1 AUTHOR
