@@ -1,14 +1,24 @@
 package Net::XMPP2::Ext;
 use warnings;
 use strict;
+use Net::XMPP2::Event;
+
+our @ISA = qw/Net::XMPP2::Event/;
 
 =head1 NAME
 
-Net::XMPP2::Ext - The set of extensions for Net::XMPP2
+Net::XMPP2::Ext - Extension baseclass and documentation
 
 =head1 DESCRIPTION
 
-This module only has documentation about the supported extensions.
+This module has documentation about the supported extensions
+and also is a base class for all extensions that can be added
+via the C<add_extension> method of the classes that derive from
+L<Net::XMPP2::Extendable>. (That are: L<Net::XMPP2::Client>,
+L<Net::XMPP2::Connection> and L<Net::XMPP2::IM::Connection>)
+
+Basically C<add_extension> makes the extension an event receiver
+for all events that the extended object receives.
 
 =head1 Supportet extensions
 
@@ -16,6 +26,13 @@ This is the list of supported XMPP extensions:
 
 =over 4
 
+=item XEP-0004 - Data Forms
+
+This extension handles data forms as described in XEP-0004.
+L<Net::XMPP2::Ext::DataForm> allows you to construct, receive and
+answer data forms. This is neccessary for all sorts of things in XMPP.
+For example XEP-0055 (Jabber Search) or also In-band registration.
+
 =item XEP-0086 - Error Condition Mappings
 
    "A mapping to enable legacy entities to correctly handle errors from XMPP-aware entities."
@@ -36,8 +53,6 @@ method of L<Net::XMPP2::Connection>.
 
 =back
 
-=cut
-
 =head1 AUTHOR
 
 Robin Redeker, C<< <elmex at ta-sa.org> >>, JID: C<< <elmex at jabber.org> >>
